#include "SudokuGrid.h"

#include <windows.h>
#include <gl\gl.h>
#include <gl\glu.h>
#include <gl\glut.h>
#include <stdio.h>			// Header File For Standard Input/Output
#include <stdarg.h>			// Header File For Variable Argument Routines
#include <iostream>
using namespace std;

HGLRC hRC=NULL;
HDC hDC=NULL;
HWND hWnd=NULL;
HINSTANCE hInstance;
GLuint	base, subbase;				// Base Display List For The Font Set
bool keys[256];
bool active=TRUE;
bool fullscreen=FALSE;
bool complete=false;
int	mouse_x, mouse_y, square_x, square_y;           // The Current Position (actual and relative) Of The Mouse
int difficulty=0;
SudokuGrid s;

LRESULT	CALLBACK WndProc(HWND, UINT, WPARAM, LPARAM);
GLvoid BuildFont(GLvoid) {
	HFONT	font;										// Windows Font ID
	HFONT	oldfont;									// Used For Good House Keeping

	base = glGenLists(96);								// Storage For 96 Characters
	subbase = glGenLists(96);								// Storage For 96 Characters

	font = CreateFont(	-24,							// Height Of Font
						0,								// Width Of Font
						0,								// Angle Of Escapement
						0,								// Orientation Angle
						FW_BOLD,						// Font Weight
						FALSE,							// Italic
						FALSE,							// Underline
						FALSE,							// Strikeout
						ANSI_CHARSET,					// Character Set Identifier
						OUT_TT_PRECIS,					// Output Precision
						CLIP_DEFAULT_PRECIS,			// Clipping Precision
						ANTIALIASED_QUALITY,			// Output Quality
						FF_DONTCARE|DEFAULT_PITCH,		// Family And Pitch
						"Courier New");					// Font Name

	oldfont = (HFONT)SelectObject(hDC, font);           // Selects The Font We Want
	wglUseFontBitmaps(hDC, 32, 96, base);				// Builds 96 Characters Starting At Character 32
	SelectObject(hDC, oldfont);							// Selects The Font We Want
	DeleteObject(font);									// Delete The Font

	font = CreateFont(	-12,							// Height Of Font
						0,								// Width Of Font
						0,								// Angle Of Escapement
						0,								// Orientation Angle
						FW_BOLD,						// Font Weight
						FALSE,							// Italic
						FALSE,							// Underline
						FALSE,							// Strikeout
						ANSI_CHARSET,					// Character Set Identifier
						OUT_TT_PRECIS,					// Output Precision
						CLIP_DEFAULT_PRECIS,			// Clipping Precision
						ANTIALIASED_QUALITY,			// Output Quality
						FF_DONTCARE|DEFAULT_PITCH,		// Family And Pitch
						"Courier New");					// Font Name

	oldfont = (HFONT)SelectObject(hDC, font);           // Selects The Font We Want
	wglUseFontBitmaps(hDC, 32, 96, subbase);				// Builds 96 Characters Starting At Character 32
	SelectObject(hDC, oldfont);							// Selects The Font We Want
	DeleteObject(font);									// Delete The Font
}

GLvoid KillFont(GLvoid)	{
	glDeleteLists(base, 96);							// Delete All 96 Characters
}

GLvoid glPrint(const char *fmt, ...) {
	char		text[256];								// Holds Our String
	va_list		ap;										// Pointer To List Of Arguments

	if (fmt == NULL)									// If There's No Text
		return;											// Do Nothing

	va_start(ap, fmt);									// Parses The String For Variables
	    vsprintf(text, fmt, ap);						// And Converts Symbols To Actual Numbers
	va_end(ap);											// Results Are Stored In Text

	glPushAttrib(GL_LIST_BIT);							// Pushes The Display List Bits
	glListBase(base - 32);								// Sets The Base Character to 32
	glCallLists((GLsizei)strlen(text), GL_UNSIGNED_BYTE, text);	// Draws The Display List Text
	glPopAttrib();										// Pops The Display List Bits
}

GLvoid glPrintSmall(const char *fmt, ...) {
	char		text[256];								// Holds Our String
	va_list		ap;										// Pointer To List Of Arguments

	if (fmt == NULL)									// If There's No Text
		return;											// Do Nothing

	va_start(ap, fmt);									// Parses The String For Variables
	    vsprintf(text, fmt, ap);						// And Converts Symbols To Actual Numbers
	va_end(ap);											// Results Are Stored In Text

	glPushAttrib(GL_LIST_BIT);							// Pushes The Display List Bits
	glListBase(subbase - 32);								// Sets The Base Character to 32
	glCallLists((GLsizei)strlen(text), GL_UNSIGNED_BYTE, text);	// Draws The Display List Text
	glPopAttrib();										// Pops The Display List Bits
}

GLvoid ReSizeGLScene(GLsizei width, GLsizei height) {	// Resize And Initialize The GL Window
	if (height==0) {	// Prevent A Divide By Zero By
		height=1;		// Making Height Equal One
	}

	glViewport(0, 0, width, height);	// Reset The Current Viewport
	glMatrixMode(GL_PROJECTION);		// Select The Projection Matrix
	glLoadIdentity();					// Reset The Projection Matrix

	// Calculate The Aspect Ratio Of The Window
	gluPerspective(45.0f,(GLfloat)width/(GLfloat)height,0.1f,100.0f);

	glMatrixMode(GL_MODELVIEW);			// Select The Modelview Matrix
	glLoadIdentity();					// Reset The Modelview Matrix
}
int InitGL(GLvoid) {
	glShadeModel(GL_SMOOTH);
	glClearColor(1.0f, 1.0f, 1.0f, 0.0f);
	glClearDepth(1.0f);
	glEnable(GL_DEPTH_TEST);
	glDepthFunc(GL_LEQUAL);
	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);

	BuildFont();

	return TRUE;
}

int DrawGLScene(GLvoid)	{
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glLoadIdentity();									// Reset The Current Modelview Matrix
	glColor3f(0.0f, 0.0f, 0.0f);
	glPushMatrix();
		//glTranslatef(-1.5f,0.0f,-6.0f);
		for (int i=0;i<10;i++) {
			if (i%3==0) {
				glLineWidth(3.0f);
			} else {
				glLineWidth(1.0f);
			}
			glBegin(GL_LINES);
				glVertex3f(-2.25+i*0.5f,2.26,-6);
				glVertex3f(-2.25+i*0.5f,-2.26,-6);
			glEnd();
		}
		for (int i=0;i<10;i++) {
			if (i%3==0) {
				glLineWidth(3.0f);
			} else {
				glLineWidth(1.0f);
			}
			glBegin(GL_LINES);
				glVertex3f(2.26,-2.25+i*0.5f,-6);
				glVertex3f(-2.26,-2.25+i*0.5f,-6);
			glEnd();
		}
	glPopMatrix();
	glPushMatrix();
		glTranslatef(0.0f,0.0f,-1.0f);						// Move One Unit Into The Screen
		//glColor3f(0.0f, 0.0f, 0.0f);
		for (unsigned int i=0; i<9; i++) {
			for (unsigned int j=0; j<9; j++) {
				unsigned int val = s.getProgress(i,j);
				if (val != 0) {   // Position The Text On The Screen
					glRasterPos2f(-0.345+j*0.0835f, 0.32-i*0.083f);
	 				glPrint("%1.0d", val);	// Print GL Text To The Screen
				}
			}
		}
	glPopMatrix();
	glPushMatrix();
		glTranslatef(0.0f,0.0f,-1.0f);			// Move One Unit Into The Screen
		//glColor3f(0.0f, 0.0f, 0.0f);
		for (unsigned int i=0; i<9; i++) {
			for (unsigned int j=0; j<9; j++) {
				unsigned int val = s.getProgress(i,j);
				if (val == 0) {
					if (s.getDepth(i,j,1)) {	// Position The Text On The Screen
						glRasterPos2f(-0.345+j*0.0835f-0.022f, 0.325-i*0.083f+0.0276f);
	 					glPrintSmall("1");	// Print GL Text To The Screen
					}
					if (s.getDepth(i,j,2)) {	// Position The Text On The Screen
						glRasterPos2f(-0.345+j*0.0835f+0.005f, 0.325-i*0.083f+0.0276f);
	 					glPrintSmall("2");	// Print GL Text To The Screen
					}
					if (s.getDepth(i,j,3)) {	// Position The Text On The Screen
						glRasterPos2f(-0.345+j*0.0835f+0.030f, 0.325-i*0.083f+0.0276f);
	 					glPrintSmall("3");	// Print GL Text To The Screen
					}
					if (s.getDepth(i,j,4)) {	// Position The Text On The Screen
						glRasterPos2f(-0.345+j*0.0835f-0.022f, 0.325-i*0.083f);
	 					glPrintSmall("4");	// Print GL Text To The Screen
					}
					if (s.getDepth(i,j,5)) {	// Position The Text On The Screen
						glRasterPos2f(-0.345+j*0.0835f+0.005f, 0.325-i*0.083f);
	 					glPrintSmall("5");	// Print GL Text To The Screen
					}
					if (s.getDepth(i,j,6)) {	// Position The Text On The Screen
						glRasterPos2f(-0.345+j*0.0835f+0.030f, 0.325-i*0.083f);
	 					glPrintSmall("6");	// Print GL Text To The Screen
					}
					if (s.getDepth(i,j,7)) {	// Position The Text On The Screen
						glRasterPos2f(-0.345+j*0.0835f-0.022f, 0.325-i*0.083f-0.0276f);
	 					glPrintSmall("7");	// Print GL Text To The Screen
					}
					if (s.getDepth(i,j,8)) {	// Position The Text On The Screen
						glRasterPos2f(-0.345+j*0.0835f+0.005f, 0.325-i*0.083f-0.0276f);
	 					glPrintSmall("8");	// Print GL Text To The Screen
					}
					if (s.getDepth(i,j,9)) {	// Position The Text On The Screen
						glRasterPos2f(-0.345+j*0.0835f+0.030f, 0.325-i*0.083f-0.0276f);
	 					glPrintSmall("9");	// Print GL Text To The Screen
					}
				}
			}
		}
	glPopMatrix();
	return TRUE;
}
GLvoid KillGLWindow(GLvoid) {
	if (fullscreen) {
		ChangeDisplaySettings(NULL,0);
		ShowCursor(TRUE);
	}
	if (hRC) {
		if (!wglMakeCurrent(NULL,NULL)) {
			MessageBox(NULL,"Release Of DC And RC Failed.","SHUTDOWN ERROR",MB_OK | MB_ICONINFORMATION);
		}
		if (!wglDeleteContext(hRC)) {
			MessageBox(NULL,"Release Rendering Context Failed.","SHUTDOWN ERROR",MB_OK | MB_ICONINFORMATION);
		}
		hRC=NULL;
	}
	if (hDC && !ReleaseDC(hWnd,hDC)) {
		MessageBox(NULL,"Release Device Context Failed.","SHUTDOWN ERROR",MB_OK | MB_ICONINFORMATION);
		hDC=NULL;
	}
	if (hWnd && !DestroyWindow(hWnd)) {
		MessageBox(NULL,"Could Not Release hWnd.","SHUTDOWN ERROR",MB_OK | MB_ICONINFORMATION);
		hWnd=NULL;
	}
	if (!UnregisterClass("OpenGL",hInstance)) {
		MessageBox(NULL,"Could Not Unregister Class.","SHUTDOWN ERROR",MB_OK | MB_ICONINFORMATION);
		hInstance=NULL;
	}

	KillFont();
}
BOOL CreateGLWindow(char* title, int width, int height, int bits, bool fullscreenflag) {
	GLuint		PixelFormat;
	WNDCLASS	wc;
	DWORD		dwExStyle;
	DWORD		dwStyle;
	RECT WindowRect;
	WindowRect.left=(long)0;
	WindowRect.right=(long)width;
	WindowRect.top=(long)0;
	WindowRect.bottom=(long)height;
	fullscreen=fullscreenflag;
	hInstance		= GetModuleHandle(NULL);
	wc.style		= CS_HREDRAW | CS_VREDRAW | CS_OWNDC;
	wc.lpfnWndProc	= (WNDPROC) WndProc;
	wc.cbClsExtra	= 0;
	wc.cbWndExtra	= 0;
	wc.hInstance	= hInstance;
	wc.hIcon		= LoadIcon(NULL, IDI_WINLOGO);
	wc.hCursor		= LoadCursor(NULL, IDC_ARROW);
	wc.hbrBackground	= NULL;
	wc.lpszMenuName		= NULL;
	wc.lpszClassName	= "OpenGL";
	if (!RegisterClass(&wc)) {
		MessageBox(NULL,"Failed To Register The Window Class.","ERROR",MB_OK|MB_ICONEXCLAMATION);
		return FALSE;
	}
	if (fullscreen) {
		DEVMODE dmScreenSettings;					// Device Mode
		memset(&dmScreenSettings,0,sizeof(dmScreenSettings));		// Makes Sure Memory's Cleared
		dmScreenSettings.dmSize=sizeof(dmScreenSettings);		// Size Of The Devmode Structure
		dmScreenSettings.dmPelsWidth	= width;			// Selected Screen Width
		dmScreenSettings.dmPelsHeight	= height;			// Selected Screen Height
		dmScreenSettings.dmBitsPerPel	= bits;				// Selected Bits Per Pixel
		dmScreenSettings.dmFields=DM_BITSPERPEL|DM_PELSWIDTH|DM_PELSHEIGHT;
		// Try To Set Selected Mode And Get Results.  NOTE: CDS_FULLSCREEN Gets Rid Of Start Bar.
		if (ChangeDisplaySettings(&dmScreenSettings,CDS_FULLSCREEN)!=DISP_CHANGE_SUCCESSFUL) {
			// If The Mode Fails, Offer Two Options.  Quit Or Run In A Window.
			if (MessageBox(NULL,"The Requested Fullscreen Mode Is Not Supported By\nYour Video Card. Use Windowed Mode Instead?","NeHe GL",MB_YESNO|MB_ICONEXCLAMATION)==IDYES) {
				fullscreen=FALSE;				// Select Windowed Mode (Fullscreen=FALSE)
			} else {
				// Pop Up A Message Box Letting User Know The Program Is Closing.
				MessageBox(NULL,"Program Will Now Close.","ERROR",MB_OK|MB_ICONSTOP);
				return FALSE;
			}
		}
	}
	if (fullscreen) {
		dwExStyle=WS_EX_APPWINDOW;
		dwStyle=WS_POPUP;
		ShowCursor(FALSE);
	} else {
		dwExStyle=WS_EX_APPWINDOW | WS_EX_WINDOWEDGE;
		dwStyle=WS_OVERLAPPEDWINDOW;
	}
	AdjustWindowRectEx(&WindowRect, dwStyle, FALSE, dwExStyle);
	if (!(hWnd=CreateWindowEx(	dwExStyle,				// Extended Style For The Window
		"OpenGL",				// Class Name
		title,					// Window Title
		WS_CLIPSIBLINGS |			// Required Window Style
		WS_CLIPCHILDREN |			// Required Window Style
		dwStyle,				// Selected Window Style
		0, 0,					// Window Position
		WindowRect.right-WindowRect.left,	// Calculate Adjusted Window Width
		WindowRect.bottom-WindowRect.top,	// Calculate Adjusted Window Height
		NULL,					// No Parent Window
		NULL,					// No Menu
		hInstance,				// Instance
		NULL)))	{				// Don't Pass Anything To WM_CREATE
			KillGLWindow();							// Reset The Display
			MessageBox(NULL,"Window Creation Error.","ERROR",MB_OK|MB_ICONEXCLAMATION);
			return FALSE;							// Return FALSE
		}
		static	PIXELFORMATDESCRIPTOR pfd=	{				// pfd Tells Windows How We Want Things To Be
			sizeof(PIXELFORMATDESCRIPTOR),					// Size Of This Pixel Format Descriptor
				1,								// Version Number
				PFD_DRAW_TO_WINDOW |						// Format Must Support Window
				PFD_SUPPORT_OPENGL |						// Format Must Support OpenGL
				PFD_DOUBLEBUFFER,						// Must Support Double Buffering
				PFD_TYPE_RGBA,							// Request An RGBA Format
				bits,								// Select Our Color Depth
				0, 0, 0, 0, 0, 0,						// Color Bits Ignored
				0,								// No Alpha Buffer
				0,								// Shift Bit Ignored
				0,								// No Accumulation Buffer
				0, 0, 0, 0,							// Accumulation Bits Ignored
				16,								// 16Bit Z-Buffer (Depth Buffer)
				0,								// No Stencil Buffer
				0,								// No Auxiliary Buffer
				PFD_MAIN_PLANE,							// Main Drawing Layer
				0,								// Reserved
				0, 0, 0								// Layer Masks Ignored
		};
		if (!(hDC=GetDC(hWnd)))	{						// Did We Get A Device Context?
			KillGLWindow();							// Reset The Display
			MessageBox(NULL,"Can't Create A GL Device Context.","ERROR",MB_OK|MB_ICONEXCLAMATION);
			return FALSE;							// Return FALSE
		}
		if (!(PixelFormat=ChoosePixelFormat(hDC,&pfd)))	{			// Did Windows Find A Matching Pixel Format?
			KillGLWindow();							// Reset The Display
			MessageBox(NULL,"Can't Find A Suitable PixelFormat.","ERROR",MB_OK|MB_ICONEXCLAMATION);
			return FALSE;							// Return FALSE
		}
		if(!SetPixelFormat(hDC,PixelFormat,&pfd)) {				// Are We Able To Set The Pixel Format?
			KillGLWindow();							// Reset The Display
			MessageBox(NULL,"Can't Set The PixelFormat.","ERROR",MB_OK|MB_ICONEXCLAMATION);
			return FALSE;							// Return FALSE
		}
		if (!(hRC=wglCreateContext(hDC))) {					// Are We Able To Get A Rendering Context?
			KillGLWindow();							// Reset The Display
			MessageBox(NULL,"Can't Create A GL Rendering Context.","ERROR",MB_OK|MB_ICONEXCLAMATION);
			return FALSE;							// Return FALSE
		}
		if(!wglMakeCurrent(hDC,hRC)) {					// Try To Activate The Rendering Context
			KillGLWindow();							// Reset The Display
			MessageBox(NULL,"Can't Activate The GL Rendering Context.","ERROR",MB_OK|MB_ICONEXCLAMATION);
			return FALSE;							// Return FALSE
		}
		ShowWindow(hWnd,SW_SHOW);						// Show The Window
		SetForegroundWindow(hWnd);						// Slightly Higher Priority
		SetFocus(hWnd);								// Sets Keyboard Focus To The Window
		ReSizeGLScene(width, height);
		if (!InitGL()) {							// Initialize Our Newly Created GL Window
			KillGLWindow();							// Reset The Display
			MessageBox(NULL,"Initialization Failed.","ERROR",MB_OK|MB_ICONEXCLAMATION);
			return FALSE;							// Return FALSE
		}
		return TRUE;								// Success
}

LRESULT CALLBACK WndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
	switch (uMsg) {
			case WM_ACTIVATE: {						// Watch For Window Activate Message
				if (!HIWORD(wParam)) {					// Check Minimization State
					active=TRUE;					// Program Is Active
				} else {
					active=FALSE;					// Program Is No Longer Active
				}

				return 0;						// Return To The Message Loop
							  }
			case WM_SYSCOMMAND: {						// Intercept System Commands
				switch (wParam)	{					// Check System Calls
			case SC_SCREENSAVE:				// Screensaver Trying To Start?
			case SC_MONITORPOWER:				// Monitor Trying To Enter Powersave?
				return 0;					// Prevent From Happening
				}
				break;							// Exit
								}
			case WM_CLOSE: {							// Did We Receive A Close Message?
				PostQuitMessage(0);					// Send A Quit Message
				return 0;						// Jump Back
						   }
			case WM_KEYDOWN: {						// Is A Key Being Held Down?
				keys[wParam] = TRUE;					// If So, Mark It As TRUE
				return 0;						// Jump Back
							 }
			case WM_KEYUP: {							// Has A Key Been Released?
				keys[wParam] = FALSE;					// If So, Mark It As FALSE
				return 0;						// Jump Back
						   }
			case WM_SIZE: {							// Resize The OpenGL Window
				ReSizeGLScene(LOWORD(lParam),HIWORD(lParam));		// LoWord=Width, HiWord=Height
				return 0;						// Jump Back
						  }

			case WM_LBUTTONDOWN: {
				if (!complete) {
					mouse_x = LOWORD(lParam);          
					mouse_y = HIWORD(lParam);
					square_x = (mouse_x<24?-1:(mouse_x>456?-1:(mouse_x-24)/48));
					square_y = (mouse_y<24?-1:(mouse_y>456?-1:(mouse_y-24)/48));
					if (square_x==-1 || square_y==-1)
						break;
					if (s.getProgress(square_y, square_x)!=0)
						break;
					int inner_x = ((mouse_x-24)%48)/16;
					int inner_y = ((mouse_y-24)%48)/16;
					int depth = 1+inner_x+(inner_y*3);
					if (!s.getDepth(square_y, square_x, depth))
						break;
					s.setSpot(square_y, square_x, depth);
					complete = s.checkComplete();
					//cout << "Left " << inner_x << " " << inner_y << " = " << depth;
					//cout << " =>" << square_x << " " << square_y << endl;
				} 
				if (complete) {
					if (MessageBox(NULL, "Play Again?", "Congratulations, you won!", MB_YESNO)==7) {
						PostQuitMessage(0);					// Send A Quit Message
						return 0;						// Jump Back
					} else {
						s.newGame();
						complete = false;
					}
				}
				}
				break;

			case WM_MBUTTONDOWN: {
				if (!complete) {
					mouse_x = LOWORD(lParam);          
					mouse_y = HIWORD(lParam);
					square_x = (mouse_x<24?-1:(mouse_x>456?-1:(mouse_x-24)/48));
					square_y = (mouse_y<24?-1:(mouse_y>456?-1:(mouse_y-24)/48));
					if (square_x==-1 || square_y==-1)
						break;
					else if (s.checkIfInitial(square_y, square_x))
						break;
					else if (s.getProgress(square_y, square_x)!=0)
						s.setSpot(square_y, square_x, 0);
					else
						s.resetChoices(square_y, square_x);
					//cout << "Middle " << mouse_x << " " << mouse_y << endl;
				}}
				break;

			case WM_RBUTTONDOWN: {
				if (!complete) {
					mouse_x = LOWORD(lParam);          
					mouse_y = HIWORD(lParam);
					square_x = (mouse_x<24?-1:(mouse_x>456?-1:(mouse_x-24)/48));
					square_y = (mouse_y<24?-1:(mouse_y>456?-1:(mouse_y-24)/48));
					if (square_x==-1 || square_y==-1)
						break;
					if (s.getProgress(square_y, square_x)!=0)
						break;
					int inner_x = ((mouse_x-24)%48)/16;
					int inner_y = ((mouse_y-24)%48)/16;
					int depth = 1+inner_x+(inner_y*3);
					s.narrowChoice(square_y, square_x, depth);
					//cout << "Right " << mouse_x << " " << mouse_y << endl;
				}
				}
				break;

			//case WM_MOUSEMOVE: {
			//		mouse_x = LOWORD(lParam);          
			//		mouse_y = HIWORD(lParam);
			//	}
			//	break;
	}
	// Pass All Unhandled Messages To DefWindowProc
	return DefWindowProc(hWnd,uMsg,wParam,lParam);
}

int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow) {
	MSG		msg;									// Windows Message Structure
	BOOL	done=FALSE;								// Bool Variable To Exit Loop

	// Ask The User Which Screen Mode They Prefer
	//if (MessageBox(NULL,"Would You Like To Run In Fullscreen Mode?", "Start FullScreen?",MB_YESNO|MB_ICONQUESTION)==IDNO) {
		fullscreen=FALSE;							// Windowed Mode
	//}


	// Create Our OpenGL Window
	if (!CreateGLWindow("Sudoku v2.0",480,480,16,fullscreen)) {
		return 0;									// Quit If Window Was Not Created
	}

	while(!done) {
		Sleep(1000/60);
		if (PeekMessage(&msg,NULL,0,0,PM_REMOVE)) {
			if (msg.message==WM_QUIT) {
				done=TRUE;							// If So done=TRUE
			} else {
				TranslateMessage(&msg);				// Translate The Message
				DispatchMessage(&msg);				// Dispatch The Message
			}
		} else {
			// Draw The Scene.  Watch For ESC Key And Quit Messages From DrawGLScene()
			if (active) {
				if (keys[VK_ESCAPE]) {
					done=TRUE;						// ESC Signalled A Quit
				} else {
					DrawGLScene();					// Draw The Scene
					SwapBuffers(hDC);				// Swap Buffers (Double Buffering)
				}
			}

			if (keys[VK_F1]) {
				keys[VK_F1]=FALSE;					// If So Make Key FALSE
				KillGLWindow();						// Kill Our Current Window
				fullscreen=!fullscreen;				// Toggle Fullscreen / Windowed Mode
				// Recreate Our OpenGL Window
				if (!CreateGLWindow("NeHe's OpenGL Framework",480,480,16,fullscreen)) {
					return 0;						// Quit If Window Was Not Created
				}
			}
		}
	}

	// Shutdown
	KillGLWindow();									// Kill The Window
	return ((int)msg.wParam);							// Exit The Program
}

int main() {
	//s.displaySol();
	//s.display();

	HINSTANCE hPrevInstance=hInstance;
	LPSTR lpCmdLine="";
	int nCmdShow=0;
	WinMain(hInstance, hPrevInstance, lpCmdLine, nCmdShow);
	return 0;
}